def unweighted_continuous_kmeans_price_of_hierarchy(
    points: list[tuple[float, list[float]]],
) -> float: ...
def weighted_continuous_kmeans_price_of_hierarchy(
    weighted_points: list[list[float]],
) -> float: ...
def unweighted_discrete_kmeans_price_of_hierarchy(
    points: list[tuple[float, list[float]]],
) -> float: ...
def weighted_discrete_kmeans_price_of_hierarchy(
    weighted_points: list[list[float]],
) -> float: ...
def unweighted_discrete_kmedian_price_of_hierarchy(
    points: list[tuple[float, list[float]]],
) -> float: ...
def weighted_discrete_kmedian_price_of_hierarchy(
    weighted_points: list[list[float]],
) -> float: ...
def unweighted_continuous_kmeans_price_of_greedy(
    points: list[tuple[float, list[float]]],
) -> float: ...
def weighted_continuous_kmeans_price_of_greedy(
    weighted_points: list[list[float]],
) -> float: ...
def unweighted_discrete_kmeans_price_of_greedy(
    points: list[tuple[float, list[float]]],
) -> float: ...
def weighted_discrete_kmeans_price_of_greedy(
    weighted_points: list[list[float]],
) -> float: ...
def unweighted_discrete_kmedian_price_of_greedy(
    points: list[tuple[float, list[float]]],
) -> float: ...
def weighted_discrete_kmedian_price_of_greedy(
    weighted_points: list[list[float]],
) -> float: ...
